##
## This script contains R code to further process and visualise the results
## produced by the sciClone software (Miller et al., 2014).
##
## run script once for each patient,  best run via:
##  source("process_sciclone_output.R")

## ----------------------------------------------------------------------------------------------------------
## Run parameters (edit before sourcing the script!)
## ----------------------------------------------------------------------------------------------------------

thisPat <- "CB1001"
## threshold for considering a clone to be present:
minFrac <- 0.05


## ----------------------------------------------------------------------------------------------------------
## Load required R packages
## ----------------------------------------------------------------------------------------------------------

library("RColorBrewer")
library("S4Vectors")
library("xlsx")
library("reshape2")
library("ggplot2")
library("clonevol")
library("extrafont")
loadfonts()


##----------------------------------------------------------------------------------
## 1. load sample and variant annotation and list of cancer-relevant genes
##----------------------------------------------------------------------------------

## load sample annotation:
clin <- read.xlsx2(file="patient_cohort_targeted_seq_Sysmed_Feb2019.xlsx",sheetIndex=1,endRow=151)
### only used in this script to assign sample names that are coherent with all other figures

## read in data frame containing details of all variants including position, nucleotide and amino-acid
##   change, affected genes (annotated using SnpEff)
B <- read.delim("all_variants_annotation.txt", stringsAsFactors=FALSE)
## the data.frame holds a column B$Gene, which is the gene name in case of a missense/stop mutation,
##  but a string of the genomic position otherwise

## load extended list of cancer-relevant genes, in order to mark putative driver mutations in tree:
load("cancerGenes.RData")
##  loads a character vector called 'cancerGenes' into workspace

##----------------------------------------------------------------------------------
## read sciClone output file for this patient
##----------------------------------------------------------------------------------

A <- read.delim(paste0(thisPat,"_sciclone_result.txt"))
##  add short string with mutation position to each variant
A$Mutation <- paste(gsub("^chr","",A$"chr"),A$"st", sep=":")

## check distribution of VAFs in input:
A.vm  <- data.matrix(A[,grep(".vaf$", names(A))])
apply(A.vm, 2, summary)

### if any mutations could not be assigned to any cluster:
message(sum(is.na(A$cluster)), " variants dropped since their cluster was set to NA.")
A <- A[!is.na(A$cluster),]

## add gene annotation:
stopifnot(all(A$Mutation %in% B$SHORTPOS))
A$gene <- B$GENE[match(A$Mutation, B$SHORTPOS)]
## note that the gene symbol is only listed for missense/stop mutations, not for silent ones
## remove duplicated gene names:
A$gene[duplicated(A$gene)] <- ""
## if the variants gene name is in the list of known cancer genes, this missense mutation might be putative driver:
A$is.driver <- A$gene %in% cancerGenes

## count variants per clone:
varPerClone <- table(A$"cluster")

## mean VAF per clone and sample:
aMeans <- sapply(A[,grep("^CB1.+vaf$", names(A))], function(z)
    as.numeric(by(z, factor(A$cluster, levels=as.character(sort(unique(A$cluster)))), FUN=mean)))


ACCF <- cbind(data.frame(2*aMeans), cluster=sort(unique(A$cluster)))
## CCF is estimated as two times the VAF
names(ACCF) <- gsub(".vaf$","", names(ACCF))


##----------------------------------------------------------------------------------
## derive a potential phylogeny from sciClone table
##----------------------------------------------------------------------------------

# a. pick all clones present in all samples -> stem
# b. check for clones present in subgroups of samples -> branches
# c. check for clones specific in certain samples -> leafs

A2 <- A

## use names with prepended "clone" consistently
names(varPerClone) <- paste0("clone", names(varPerClone)) 

## compute for each cluster in how many samples it is "present"
acf <- data.matrix(ACCF[,-ncol(ACCF)])
rownames(acf) <- paste0("clone", ACCF$cluster)

nSamples <- ncol(acf)
nClones <- nrow(acf)

acf.sums <- rowSums(acf >= minFrac)

## stem: common clones in all samples:
## is there a founder clone? monoclonal vs. poly-clonal development
if (max(acf.sums)==nSamples){
  stemClones <- names(acf.sums)[acf.sums==nSamples]
  stemCloneIds <- as.integer(gsub("^clone", "", stemClones))
  stemLen    <- sum(varPerClone[stemCloneIds])
  noFounder <- FALSE
} else { # if no founder clone identified
  stemClones <- "0"
  stemCloneIds <- 0L
  stemLen <- 0
  noFounder <- TRUE
}

A2$cluster[A2$cluster %in% stemCloneIds] <- min(stemCloneIds)

treeTab <- data.frame(clone=min(stemCloneIds), parent=-1L,
   sample.with.nonzero.cell.frac.ci=paste(colnames(acf),collapse=","),
   events=stemLen)


varPerClone2 <- varPerClone
varPerClone2[ stemCloneIds ] <- 0
varPerClone2[ min(stemCloneIds) ] <- sum(varPerClone[stemCloneIds])


## iterate over subsets:
## set lowest super-clone set for each clone to stem:

superSetClone  <- rep(min(stemCloneIds), nClones)
superSetNSamples <- rep(nSamples, nClones)

sampleSetsPerClone <- apply(acf, 1, function(z) which(z>=minFrac))

for (i in 1:(length(sampleSetsPerClone)-1)){
  s1 <- sampleSetsPerClone[[i]]
  if (length(s1)==nSamples) next # stem already treated
  if (length(s1)==0L){ ## too low VAFs:
     A2 <- A2[A2$cluster!=i,]
     varPerClone2[[i]] <- 0
     next
  }
  for (j in (i+1):length(sampleSetsPerClone)){
      s2 <- sampleSetsPerClone[[j]]
      if (length(s2)==nSamples) next  # stem already treated
      if (length(s2)==0L){ ## too low VAFs:
        A2 <- A2[A2$cluster!=j,]
        varPerClone2[[j]] <- 0
        next
      }
      # is s2 a subset of s1 and/or the other way around??
      s2ins1 <- all(is.element(s2, s1))
      s1ins2 <- all(is.element(s1, s2))
      # option 1: they are equal:
      if (s2ins1 && s1ins2){
         ## merge clone j into clone i
         A2$cluster[A2$cluster==j] <- i
         varPerClone2[i] <- varPerClone2[i] + varPerClone2[j] 
         varPerClone2[j] <- 0
      }
      # option 2: s2 is a true subset of s1 and s1 is smaller than the current super-set of s2
      if (s2ins1 && !s1ins2 && (length(s1)<superSetNSamples[j])){
         superSetNSamples[j] <- length(s1)
         superSetClone[j] <- i
      }
      # option 3: s1 is a true subset of s2 and s2 is smaller than the current super-set of s1
      if (!s2ins1 && s1ins2 && (length(s2)<superSetNSamples[i])){
         superSetNSamples[i] <- length(s2)
         superSetClone[i] <- j
      }           
  }# for j
}# for i


### expand tree table by non-stem clones:
for (i in 1:length(sampleSetsPerClone)){
   if ((i %in% stemCloneIds) || varPerClone2[[i]]==0) next
   treeTab <- rbind(treeTab,
     data.frame(clone=i, parent=superSetClone[[i]],
                sample.with.nonzero.cell.frac.ci=paste(names(sampleSetsPerClone[[i]]),collapse=","),
                events=varPerClone2[[i]]))
}


### cluster numbers must be integers from 1:n, replace accordingly:
oldCloneNames <- treeTab$clone
treeTab$clone <- 1L:length(oldCloneNames)
treeTab$parent <- match(treeTab$parent, oldCloneNames)
treeTab$parent[[1]] <- -1L

stopifnot(all(A2$cluster %in% oldCloneNames))
A2$cluster <- match(A2$cluster, oldCloneNames)

## export variants and tree table for import tree:
A2$cluster <- A2$cluster

varFile <- paste0("~/temp/", thisPat, "_variants.txt")
write.table(A2[,c("cluster", "gene", "is.driver", grep("CB10", names(A2), value=TRUE))],
      file=varFile, sep="\t", col.names=TRUE, row.names=FALSE, quote=FALSE)


## for tree-table make sure that only leaves have labels:
areLeafs <- !treeTab$clone %in% treeTab$parent
leafSamples <- sort(unlist(strsplit(treeTab$sample.with.nonzero.cell.frac.ci[areLeafs],",")))

## shorten non-leaf labels by removing samples labelled further down in the tree:
swn <- treeTab$sample.with.nonzero.cell.frac.ci
treatedSamples <- leafSamples
for (i in rev(which(!areLeafs))){
   remSamples <- setdiff(unlist(strsplit(swn[[i]],",")), treatedSamples)
   treatedSamples <- c(treatedSamples, remSamples)
   swn[[i]] <- paste(remSamples, collapse=",")
}
treeTab$sample.with.nonzero.cell.frac.ci <- swn


## replace sample names by shorter versions with time:
oldSampleNames <- colnames(acf)
ma1 <- match(paste0(gsub("\\.", "-", oldSampleNames),"-D01-X1"), clin$ChariteLibID)
stopifnot(!any(is.na(ma1)))

CL.PAT <- clin[ma1, c("ChariteLibID", "WES","time.point.")]
newSampleNames <- paste0(CL.PAT$"time.point.", gsub(paste0("^", thisPat, "."), "", oldSampleNames))
names(newSampleNames) <- oldSampleNames


splitted <- strsplit(treeTab$sample.with.nonzero.cell.frac.ci, ",")
treeTab$sample.with.nonzero.cell.frac.ci <-
    sapply(splitted, function(z) paste(sort(newSampleNames[z]), collapse=","))


treeTab$events <- NULL  # not needed and interferes with gene name annotation later:

## export tree file:

treeFile <- paste0(thisPat, "_trees.txt")
write.table(treeTab, file=treeFile, sep="\t", col.names=TRUE, row.names=FALSE, quote=FALSE)


## test clonevol's import.tree:
cTree1 <- import.tree(tree.file=treeFile, variant.file=varFile)
cTree1 <- convert.consensus.tree.clone.to.branch(cTree1, branch.scale = "sqrt")

## You can then also map driver events onto the tree as long as the variant file
##  you prepared has ‘cluster’,‘is.driver’, and ‘gene’ column):
if (sum(cTree1$variants$is.driver)>=1L)
  cTree1 <- transfer.events.to.consensus.trees(cTree1, cTree1$variants[cTree1$variants$is.driver,],
               cluster.col.name = 'cluster', event.col.name = 'gene')


#Now generate final tree for plotting:
CT <- cTree1$matched$merged.trees[[1]]

if (noFounder){ ## if no single founder clone found
   CT$lab <- 0:(nrow(CT)-1)
   CT$parent <- CT$parent-1L; CT$parent[[1]] <- -1L
   CT$blengths <- c(0, CT$blengths[1:(nrow(CT)-1L)])
   CT$sample.with.nonzero.cell.frac.ci[[1]] <- ""
   CT$sample.with.nonzero.cell.frac[[1]] <- ""   
}

### swap clone labels to be ordered from low to high:
for (i in 1:nrow(CT)){
   if (CT$lab[[i]] >= CT$parent[[i]]) next
   oldLab    <- CT$lab[[i]]
   oldParent <- CT$parent[[i]]
   CT$lab[CT$lab==oldParent] <- oldLab
   CT$lab[[i]] <- oldParent
   CT$parent[CT$parent==oldLab] <- oldParent
   CT$parent[CT$parent==oldParent] <- oldLab
}
CT <- CT[order(CT$lab),]

## adapt plotting parameters for each patient to improve visual clarity:
allPats <- c("CB1001","CB1002","CB1003","CB1004","CB1005",
             "CB1006","CB1007","CB1008","CB1009","CB1010")

branchWidths <- c(0.7, 0.7, 0.7, 0.6, 0.8, 0.7, 0.7, 0.36, 0.4, 0.8)
nodeSizes    <- c(4.9, 4.7, 5.4, 5.2, 5.1, 4.5, 4.5, 5.2, 5.1, 4.5)
nodeLabSizes <- c(1.9,1.6, 2.2, 2, 1.95, 1.52, 1.52, 2, 2, 1.52)
ma1 <- match(thisPat, allPats)

## generate PDF file with final tree:
picFile1 <- paste0(thisPat,"_sciclone_tree.pdf")
pdf(picFile1, width=6, height=6, useDingbats=FALSE, family="Arial")
par(mar=c(0, 0.1, 4, 0.1))
plot.tree.clone.as.branch(CT, branch.width=branchWidths[ma1], node.size=nodeSizes[ma1], node.label.size=nodeLabSizes[ma1],
  branch.text.size=nodeLabSizes[ma1], node.text.size=0.7, tree.rotation=180, angle=45, show.event=TRUE, tree.label=thisPat)
title(thisPat, cex.main=2)  
dev.off()

## finished
message("Tree for patient ", thisPat, " generated successfully and stored in file ", picFile1, ".")
